/*
Copyright or © or Copr. Abdouramane MAYAKI

mail of the author: mayakiyerima@yahoo.fr

This software is a framework for variational data assimilation in
numerical models.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/


//! \file    ConstantWindow.cpp
//! \brief   Implementation of the Constant window for visual_YAO.
//! \version 2010/10/29 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane & MAYOMBO Alexis.

/*
  Nom du Fichier: ConstantWindow.cpp
  Nom de classe: ConstantWindow
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

#include <iostream>
#include "include/ConstantWindow.hpp"
//! \class ConstantWindow(QWidget *parent): QWidget(parent);
//! \brief Implementation of the Constant window for visual_YAO.

using namespace std;
using namespace yao;

/*************************************************************** The constructor **********************************************************************/
ConstantWindow::ConstantWindow(Table <Constant> *aConstantTable, vector <string> *aHeaderList, MainWindowController *mwc, QWidget *parent): QWidget(parent), m_localConstantTable(aConstantTable), localHeaderList(aHeaderList), localMwc(mwc) { /*!< The constructor. */
	setupUi(this);	
	//m_localConstantTable 	= aConstantTable;
	//localHeaderList		= aHeaderList;
	//localMwc		= mwc;
	//localConstantController = aConstantController;
	QRegExp regExp("[A-Za-z_][A-Za-z0-9_]*"); //{0,}"); * means zero or more occurrence of the 2nd bloc. 
	QRegExp regExp2("[A-Za-z0-9_][A-Za-z0-9]*[.0-9]?[0-9]*[eE]?[+-]?[0-9]+");
	m_ligneNom->setValidator(new QRegExpValidator(regExp, this));		
	m_ligneValeur->setValidator(new QRegExpValidator(regExp2, this));

	if(isConstantTableEmpty()) m_delButton ->setDisabled(true);	  /*!< setDisabled is true. */

	m_defvalModel = new  QStandardItemModel(0, 2, this);//Nombre de lignes et colonnes du modèle.
		m_defvalModel->setHorizontalHeaderLabels(QStringList() << "Constante(s)" << "Valeur(s)"); /*!< displays the QTableWidget */

	for (Table<Constant>::iterator it = m_localConstantTable->begin(); it != m_localConstantTable->end(); it++)
		{
			m_defvalModel->setItem(m_defvalModel->rowCount(), 0, new QStandardItem(QString(it->getName().c_str())));
			m_defvalModel->setItem(m_defvalModel->rowCount()-1, 1, new QStandardItem(QString(it->getText().c_str())));//Apres le 1er ajout, la ligne vaut i+1
		}
	//if (mwc->isStringLiteral("malkj")) cout << "cest bon ça" << endl; else cout << "Oh dommage!" << endl;
	m_tableView->setModel(m_defvalModel);
	m_tableView->setEditTriggers(QAbstractItemView::NoEditTriggers); /*!< this ligne give no edit mode*/
	m_tableView->setShowGrid(false);  /*!< this ligne cover the grille in table */
	m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);  /*!< this ligne give a section of the ligne */
	m_tableView->resizeColumnsToContents();


	connect(m_helpButton, SIGNAL(clicked()), this, SLOT(callConstantHelp()));
	connect(m_quitButton, SIGNAL(clicked()), this, SLOT(close()));  /*!< slot for the close this windows*/
	connect(m_addButton, SIGNAL(clicked()), this, SLOT(addRowInTable()));  /*!< This slot add the object in the view */
	connect(m_delButton, SIGNAL(clicked()), this, SLOT(deleteRow())); /*!< To delete the object selected in the view*/
	//connect(this, SIGNAL(dataAdded()), this, SLOT(saveData())); /*!< This slot save data in structure */	
	
	setFixedWidth(baseSize().width() + 335);
	
	//	move(0, 0);
}

/*******************************************************************************************************************************************************/
/****************************************************************** addRowInTable() method *************************************************************/
void ConstantWindow::addRowInTable() {
try {  
	QString Nom  = m_ligneNom->text();
	QString Val  = m_ligneValeur->text();
	
	if (!m_localConstantTable->empty())
	for (Table<Constant>::iterator it = m_localConstantTable->begin(); it != m_localConstantTable->end(); it++)
				if (it->getName() == Nom.toStdString())
					{
						QMessageBox::critical(this, "Avertissement", "Ce nom de constante existe déjà!");
						eraseDataFromLineEdit();
						return;
					}

	if ((m_ligneNom->text().isEmpty()) || (m_ligneValeur->text().isEmpty())) {

                          QMessageBox::critical(this, "Avertissement", "Avant de cliquer sur le bouton \"Ajouter\", assurez-vous de n'avoir laissé aucun champ vide.");
                          return; // Arrêt de la méthode
                  }
  
	else {
	 	if (!(m_ligneNom->text().isEmpty()) || (m_ligneValeur->text().isEmpty())) {
	
						m_delButton->setEnabled(true);                     

			Constant *c; 

			c = m_localConstantTable->find(Val.toStdString()); 	
			if (c != NULL)
				Val	= QString(c->getText().c_str());//QString(m_localConstantTable->getText().c_str());
			else {
				
				for(int i=0; i < Val.size(); i++)
					if (!(Val[i] >= '0' && Val[i] <= '9')) {
	
						QMessageBox::critical(this, "Avertissement", "Seuls les noms des constantes defval sont acceptés sous forme de chaines.");
						return;
					}
					//else
			}

			Constant localConstant (Nom.toStdString(), Val.toStdString());	
			//for (Table<Constant>::iterator it = m_localConstantTable->begin(); it != m_localConstantTable->end(); it++)
			//	if (m_localConstantTable->size() == 0 )// it->getName().c_str() == "" /*Nom.toStdString()*/)
					
			//		{	
						//cout << it->getName() << endl;
				
			//	else {
			//		QMessageBox::critical(this, "Avertissement", "Cet élément existe déjà!");
			//		return; // Arrêt de la méthode
			//		}
		
			m_localConstantTable->push_back(localConstant);
			m_defvalModel->setItem(m_defvalModel->rowCount(), 0, new QStandardItem(Nom));
			m_defvalModel->setItem(m_defvalModel->rowCount()-1, 1, new QStandardItem(Val));//Apres le 1er ajout, la ligne vaut i+1
				
			emit rowIsAdded(); // signal emis lorsqu'une ligne a été correctement rajoutée dans le tableau. 
					   // Il sera recupéré dans le mainwidow.
		}
	}

	eraseDataFromLineEdit();
  }
  catch (...) {
	
	QMessageBox::critical(this, "Erreur", "Erreur dans la méthode addDefvalRow()");
	}
 }

/*******************************************************************************************************************************************************/
/******************************************************************* delete Row() method ***************************************************************/
void ConstantWindow::deleteRow() {
try {
	string constantNameCopy, constantValueCopie;

	//int c = m_tableView->currentIndex().row();//renvoie le numero de la ligne selectionnée.
	QItemSelectionModel *selection = m_tableView->selectionModel();
	index = selection->currentIndex();
	QVariant element = m_defvalModel->data(index, Qt::DisplayRole);
	int r = index.row();
	if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
         {
		m_defvalModel->removeRows(m_tableView->currentIndex().row(), 1);
		
		emit rowIsDeleted(element.toString()); // signal emis lorsqu'une ligne est retirée du tableau. 
		
		m_localConstantTable->erase(m_localConstantTable->begin() + r);



		string chaine = element.toString().toStdString();
		for (Table<Constant>::iterator it = m_localConstantTable->begin(); it != m_localConstantTable->end(); it++)
			{
				
				if (chaine == it->getName())
				chaine = it->getText().c_str();
			}
	     for (int i=0; i < localHeaderList->size(); i++ ) //Parcourt le vecteur de HatName en fonction de sa taille (nombre d'éléments)
		{
			
			if (element.toString().toStdString() == localHeaderList->at(i)) //Teste si l'élément à supprimer dans defval est aussi dans hat
		     		localHeaderList->erase(localHeaderList->begin() + i);//localHeaderList->begin() +i );	
		}

		if(isConstantTableEmpty()) m_delButton->setDisabled(true);
	}
   }
   catch(...) {

	QMessageBox::critical(this, "Erreur", "erreur dans la suppression d'une ligne dans deleteDefvalRow().");
	}	
}
 
/*******************************************************************************************************************************************************/
/*************************************************** eraseDataFromLineEdit() ***************************************************************************/
void ConstantWindow::eraseDataFromLineEdit() {

	m_ligneNom->setText("");
	m_ligneValeur->setText("");
}
/*******************************************************************************************************************************************************/
/*******************************************************************************************************************************************************/
void ConstantWindow::saveData() {
 
         m_quitButton->setEnabled(true);
	
}

/*******************************************************************************************************************************************************/
bool ConstantWindow::isConstantTableEmpty() {

	if (m_localConstantTable->empty())
		return true;
	else
		return false;
}
/******************************************************** callConstantHelp() ***************************************************************************/
void ConstantWindow::callConstantHelp() {

	QMessageBox::about(this, tr("Aide sur la directive Defval"),
			("<p> La directive Defval appelée aussi Constant permet d'associer un nom symbolique à une valeur. Voici les différents emplois:"
                         "<p><u>Exemple 1</u>: defval szx 12 (nombre entier douze) <p><u>Exemple 2</u>: defval szy \"chaine\" (chaine de caractères)<p> <u>Exemple 3</u>: defval szz szx (nom d'une autre constante defval)."
			 "<p>Dans la fenêtre graphique, vous devez renseigner les champs nom et valeur de la constante. Visual Yao vérifie la syntaxe " 
			 "et vous guide au fur et à mesure." 
			 "<br>Le bouton \"Ajouter\" permet d'insérer des nouveaux enregistrements (ici le nom et la valeur d'une constante) dans la structure de données); "
			 "<br>Le bouton \"Supprimer\" permet de retirer un enregistrement sélectionné de la structure de données;"
			 "<br> Le bouton \"Quitter\" permet de fermer la fenêtre;" 
			 "<br> Le bouton \"Aide\" permet d'afficher cette fenêtre."
			 "<br> Attention, tous les caractères ne sont pas acceptés dans les champs nom et valeur de la constante. Le premier ne laisse paser que les caratères [a(0-9)-z] ou [A(0-9)-Z]; et le second des chiffres ou des noms de constantes déjà déclarées."
                         "<p> Pour plus d'informations sur defval, <a href=\"help/constante_aide.html\">cliquez ici.</a>"));

}
/*******************************************************************************************************************************************************/
/*******************************************************************************************************************************************************/
void ConstantWindow::closeEvent(QCloseEvent *event) {
	
	if (!(m_ligneNom->text().isEmpty()) && !(m_ligneValeur->text().isEmpty())) { //When the two QLineEdit content data, show a confirmation box.	
	
	int ret = QMessageBox::warning(this, tr("Confirmation de fermeture"), tr("Voulez-vous quitter sans sauvegarder les données contenues dans les champs?"),
                          QMessageBox::Yes | QMessageBox::Default, QMessageBox::Cancel | QMessageBox::Escape);
	
	if (ret == QMessageBox::Yes) 
                 event->accept();
               else
                 event->ignore();

	}
        	emit windowIsClosed(1); //The signal is emitted with parameter "1" which is used for "DevalWindow". 	

	//	~ConstantWindow();
}

/*******************************************************************************************************************************************************/
/************************************************ Le destructeur ~ConstantWindow() ********************************************************************/
ConstantWindow::~ConstantWindow() {

/*	//delete m_localConstant;
        //delete localHeaderList;
	delete m_localConstantTable;
	delete m_localConstant;
	delete localHeaderList;
 
	delete m_modeleListeNoms;
	delete m_defvalModel;
	delete localMwc;
*/
	

 
}
/****************************************************************************************************************************************************/
/************************************************** The end of the file. ****************************************************************************/
/****************************************************************************************************************************************************/
