//! \file    ContextWindow.cpp
//! \brief   Implementation of the Context window for visual_YAO.
//! \version 2010/10/29 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.

/*
  Nom du Fichier: ContextWindow.cpp
  Nom de classe:
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

#include "include/ContextWindow.hpp"
using namespace std;
using namespace yao;

//! \class ContextWindow(QWidget *parent) : QWidget(parent)
//! \brief Implementation of the Context window for visual_YAO.
/******************************************************************************************************************************************************/
ContextWindow::ContextWindow(Context *aContext, Table <Constant> *aConstantTable, string *contextData, QWidget *parent) : QWidget(parent), localConstantTable(NULL), saveState(false) {

	setupUi(this);
	localConstantTable 	 = aConstantTable;
	m_localContext		 = aContext;
	m_contextParameters	 = contextData;

	if (isContextListEmpty()) m_delButtonO_EXTOBJ->setDisabled(true);  
		m_comboContextO_REAL->insertItem(0, "float"); 
		m_comboContextO_REAL->insertItem(1, "double"); 
		m_comboContextO_MQN->insertItem(0, "M1QN3"); 
		m_comboContextO_MQN->insertItem(1, "M2QN1"); 
		m_comboContextO_PARALLEL->insertItem(0, ""); 
		m_comboContextO_PARALLEL->insertItem(1, "Actif"); 
				
		m_comboContextO_NETWARD->insertItem(0, ""); 
		m_comboContextO_NETWARD->setEditable(true);
		m_comboContextO_EXTOBJ->insertItem(0, ""); 

		m_comboContextO_PARALLEL->setEditable(true);

	//m_contextSaveButton->setDisabled(true);

	for (Table<Constant>::reverse_iterator it = localConstantTable->rbegin(); it != localConstantTable->rend(); it++) 
		{	//To full the some comboboxs.
			m_comboContextO_PARALLEL->insertItem(2, QString(it->getName().c_str()));
			m_comboContextO_NETWARD->insertItem(1, QString(it->getName().c_str()));
			m_comboContextO_EXTOBJ->insertItem(1, QString(it->getName().c_str())); 
		}

	//filesListModele = new QStringListModel(filesList);

	O_EXTOBJModele = new QStandardItemModel(0, 1) ;//To manage the O_EXTOBJ's case. This is the model. 	

	for (vector <string>::reverse_iterator it = m_fileList.rbegin(); it != m_fileList.rend(); it++ ) 
		{
			O_EXTOBJModele->appendRow(new QStandardItem(QString(it->c_str())));	
		}
	
	istringstream theStream(*m_contextParameters);// To manage the string with the stream.
	string 	theStreamString;
	string theStringTab[20];
	for (int i = 0; i <= 20; i++) {
		while (getline(theStream, theStreamString, '\n' ) ) {
			theStringTab[i] = theStreamString;
		}	
		//if (theStreamString )	
	}
	
	cout << "4 element: " << theStringTab[4] << endl;
	

	m_contextO_EXTOBJListView->setModel(O_EXTOBJModele); 

	//connect(m_enregContext, SIGNAL(clicked()), this, SLOT(saveContextData()));
        connect(m_contextSaveButton, SIGNAL(clicked()), this, SLOT(saveParameters()));
	connect(m_contextResetButton, SIGNAL(clicked()), this, SLOT(resetParameters()));
        connect(m_contextQuitButton, SIGNAL(clicked()), this, SLOT(close()));
	connect(m_contextHelpButton, SIGNAL(clicked()), this, SLOT(callContextHelp()));
	
        connect(m_addButtonO_EXTOBJ, SIGNAL(clicked()), this, SLOT(addO_EXTOBJRow()));
	connect(m_delButtonO_EXTOBJ, SIGNAL(clicked()), this, SLOT(deleteO_EXTOBJRow()));

	setFixedWidth(baseSize().width() + 575);
}

/**************************************** Implémentation de la méthode saveParameters() **************************************************************/
void ContextWindow::saveParameters() {

	try 
		{
			/*	m_contextModel->setItem(row, 0, new QStandardItem());  
		
			*/			
			m_contextParameters->clear();

			string option = "option ";
			string o_real 		= m_comboContextO_REAL->currentText().toStdString(),
				o_mqn 		= m_comboContextO_MQN->currentText().toStdString(),
				o_parallel	= m_comboContextO_PARALLEL->currentText().toStdString(),
				o_netward	= m_comboContextO_NETWARD->currentText().toStdString(),	
				o_real2, o_mqn2, o_parallel2, o_netward2;	
			
			if (!o_real.empty())
				//QMessageBox::critical(this, "Erreur", "");
				if ((o_real == "float") || (o_real == "double")) 
					o_real2 = option + "O_REAL " + o_real;		
				
			if (!o_mqn.empty())
				if ((o_mqn == "M1QN3") || (o_mqn == "M2QN1"))
					o_mqn2 = option + o_mqn;
	
			if (!o_parallel.empty()) 
				{
					if ((o_parallel == "Actif")) 
						o_parallel2 = option + "O_PARALLEL" ;
					else 
						{
							Constant *c1; //Nous créons d'abord un pointeur de constante.
							c1  = localConstantTable->find(o_parallel);
							if (c1 != 0) {
								string value = localConstantTable->find(o_parallel)->getText();
								o_parallel2 = option + "O_PARALLEL " + value;
							}
							else	{
								for(unsigned int i=0; i < o_parallel.size(); i++) {
									if(!(o_parallel[i] >= '0' && o_parallel[i] <= '9')){
										QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"o_parallel\"."));     
										return;
									}
								}
								o_parallel2 = option + "o_parallel " + o_parallel;                                                 
								//theShape.push_back(toInt(d1));
							//if (o_parallel == )
							}
						}
				}
			
			if (!o_netward.empty()) 
				{
					if ((o_netward == "ACTIF")) 
						o_netward = option + "O_PARALLEL" ;
					else 
						{
							Constant *c1; //Nous créons d'abord un pointeur de constante.
							c1  = localConstantTable->find(o_netward);
							if (c1 != 0) {
								string value = localConstantTable->find(o_netward)->getText();
								o_netward2	= option + "O_NETWARD " + value;
							}
							else	{
								for(unsigned int i=0; i < o_netward.size(); i++) {
									if(!(o_netward[i] >= '0' && o_netward[i] <= '9')){
										QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"O_NETWARD\"."));     
										return;
									}
								}
								o_netward2 = option + "O_NETWARD " + o_netward;                                                 
								//theShape.push_back(toInt(d1));
							//if (o_parallel == )
							}
						}
				}

			for (vector <string>::reverse_iterator it = m_fileList.rbegin(); it != m_fileList.rend(); it++ )
				{
					string test = "option ";
					m_contextParameters->append(test.append(it->c_str()));// << endl;
					m_contextParameters->append("\n");

				}
	
			if (!o_real2.empty())
				m_contextParameters->append("\n" + o_real2) ; 
			if (!o_mqn2.empty())
				m_contextParameters->append("\n" + o_mqn2) ; 
			if (!o_parallel2.empty())
				m_contextParameters->append("\n" + o_parallel2) ;
			if (!o_netward2.empty()) 
				m_contextParameters->append("\n" + o_netward2) ;
			if (m_checkBoxO_GRADTEST->isChecked())  
				m_contextParameters->append("\noption O_GRADTEST ");
			if (m_checkBoxO_VARINCR->isChecked())  
				m_contextParameters->append("\noption O_VARINCR ");
			if (m_checkBoxO_DBG_NANF->isChecked())  
				m_contextParameters->append("\noption O_DBG_NANF ");
			if (m_checkBoxO_DBG_TING->isChecked())  
				m_contextParameters->append("\noption O_DBG_TING ");
			if (m_checkBoxO_DBG_BETA->isChecked())  
				m_contextParameters->append("\noption O_DBG_BETA ");
			if (m_checkBoxO_DBG_TRC_FWARD->isChecked())  
				m_contextParameters->append("\noption O_DBG_TRC_FWARD ");
			if (m_checkBoxO_DBG_TRC_BWARD->isChecked())  
				m_contextParameters->append("\noption O_DBG_TRC_BWARD ");
			if (m_checkBoxO_DBG_TRC_LWARD->isChecked())  
				m_contextParameters->append("\noption O_DBG_TRC_LWARD ");
			if (m_checkBoxO_DBG_TRC_AWARD->isChecked())  
				m_contextParameters->append("\noption O_DBG_TRC_AWARD ");

			m_localContext->addExternal(m_fileList);
			//string *test;
			//cout << "voici les parametres: " << *m_contextParameters << endl; 
			//eraseDataFromLineEdit();	
			m_contextSaveButton->setDisabled(true);
			saveState = true;
		}
		catch(...) 
			{
				QMessageBox::critical(this, "Erreur", "Erreur dans la methode saveParameters()");
			}
}
              
/**************************************** Implémentation de la méthode saveParameters() **************************************************************/
void ContextWindow::addO_EXTOBJRow() {

	try 
		{
			O_EXTOBJModele->appendRow(new QStandardItem(m_comboContextO_EXTOBJ->currentText())); 
			m_fileList.push_back(m_comboContextO_EXTOBJ->currentText().toStdString());	
			m_comboContextO_EXTOBJ->setCurrentIndex(0);
		}
		catch(...) 
			{
				QMessageBox::critical(this, "Erreur", "Erreur dans la methode addO_EXTOBJRow()");
			}
}

/**************************************** Implémentation de la méthode deleteO_EXTOBJRow() **********************************************************/
void ContextWindow::deleteO_EXTOBJRow() {

	QItemSelectionModel *selection = m_contextO_EXTOBJListView->selectionModel(); 
	O_EXTOBJIndex = selection->currentIndex(); 

	if (!O_EXTOBJIndex.isValid()) 
		return; 

	QVariant element = O_EXTOBJModele->data(O_EXTOBJIndex, Qt::DisplayRole);
	int r = O_EXTOBJIndex.row();

	if (selection->hasSelection()) 
		{
			O_EXTOBJModele->removeRows(m_contextO_EXTOBJListView->currentIndex().row(), 1); 
			//localHeaderList->erase(localHeaderList->begin() + r);
			//if(isHeaderListEmpty()) m_delHatButton->setDisabled(true);
		}
	//cout << "Salut" << endl;
}

/********************************** Implementation of the method getContextParameters() ************************************************************/
/*string ContextWindow::getContextParameters() {

	//if (!m_contextParameters.empty())
		return m_contextParameters;
	//else
}*/
/******************************** Implementation of the method isEmpty() *************************************************************************/
bool ContextWindow::isEmpty() {
	
	if (m_contextParameters->empty())
		return true;
	else
		return false;
	
}
/*************************************** Implémentation de la méthode resetParameters ****************************************************************/
void ContextWindow::resetParameters() {

	eraseDataFromLineEdit();	
	m_contextSaveButton->setEnabled(true);
}

/*****************************************************************************************************************************/
bool ContextWindow:: isContextListEmpty() {

	//int row = O_EXTOBJModele->rowCount();
	//if (row == 0)
	//	return true;
	//else 
		return false;
}


/*****************************************************************************************************************************/
/*****************************************************************************************************************************/
/****************************************************************************************************************************/
//QStringList ContextWindow::getContextData() {// Get the context data saved.

//	return filesList;
//}

/****************************************************************************************************************************/

/************************************* Implémentation de la méthode eraseDataFromLineEdit() ***********************************************************/
void ContextWindow::eraseDataFromLineEdit() {

	m_comboContextO_REAL->setCurrentIndex(0); 
	m_comboContextO_MQN->setCurrentIndex(0); 
	m_comboContextO_PARALLEL->setCurrentIndex(0);
	m_comboContextO_NETWARD->setCurrentIndex(0); 
	m_comboContextO_EXTOBJ->setCurrentIndex(0);
	m_checkBoxO_GRADTEST->setChecked(false);	
	m_checkBoxO_VARINCR->setChecked(false);
	m_checkBoxO_DBG_NANF->setChecked(false);
	m_checkBoxO_DBG_TING->setChecked(false);
	m_checkBoxO_DBG_BETA->setChecked(false);
	m_checkBoxO_DBG_TRC_FWARD->setChecked(false);
	m_checkBoxO_DBG_TRC_BWARD->setChecked(false);
	m_checkBoxO_DBG_TRC_LWARD->setChecked(false);
	m_checkBoxO_DBG_TRC_AWARD->setChecked(false);
		
}
/****************************************************************************************************************************/
void ContextWindow::callContextHelp() {

	QMessageBox::about(this, tr("Aide sur la directive Option."),("<p> Avec option (ou encore context), on indique les options de génération de code à appliquer; le but étant de limiter ponctuellement la taille du code généré. <br> L'exemple suivant permet de préciser le minimiseur à appliquer dans les calculs: <p><u>Exemple 1:</u> option o_m1qn3 <p> Plusieurs lignes de cette directive peuvent être déclarées consécutivement dans un même fichier de description. <p><u>Exemple 2:</u><br>...<br> option O_GRADTEST <br> option O_REAL float <br> option O_VARINCR <br>option ... <p> Pour plus d'informations sur la directive, <a href=\"help/option_aide.html\">cliquez ici.</a>"));

}

/****************************************** Implement of the method closeEvent(QCloseEvent *event) ****************************/
void ContextWindow::closeEvent(QCloseEvent *event){
                
	event->accept();
	if (!saveState) emit contextWindowIsClosed(3);//if don't save the signal to eraze data will be send.

 } 
/******************************************************************************************************************************************************/
/********************************************** End of the Context.cpp file ***************************************************************************/
/******************************************************************************************************************************************************/
