//! \file    FilesViewer.cpp
//! \brief   Implementation of the FileViewer window for visual_YAO.
//! \version 2010/10/16 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.


#include <QtGui>
#include "include/FilesViewer.hpp"

//! \class FilesViewer(QWidget *parent) : QDialog(parent)
//! \brief Implementation of the FilesViewer for visual_YAO.



/*******************************************************************************************************************************/
FilesViewer::FilesViewer(std::string projectPath, QWidget *parent) : QDialog(parent) {

	setWindowTitle(tr("Arborescence des fichiers"));

    modeleFichiers = new QDirModel;//The computer files system directories' model.
        modeleFichiers->setReadOnly(false);//Read only access.
        //modeleFichiers->setSorting(QDir::DirsFirst | QDir::IgnoreCase | QDir::Name);

    vueFichiers = new QTreeView;//The tree view.
        vueFichiers->setModel(modeleFichiers); //The view take here the model.
        vueFichiers->setRootIndex(modeleFichiers->index(QString(projectPath.c_str())));
        vueFichiers->header()->setStretchLastSection(true);
        vueFichiers->header()->setSortIndicator(0, Qt::AscendingOrder);
        vueFichiers->header()->setClickable(true);
        vueFichiers->setHeaderHidden(true);
     QModelIndex index = modeleFichiers->index(QDir::currentPath() + QString(projectPath.c_str()));
        vueFichiers->expand(index);
        vueFichiers->scrollTo(index);
        vueFichiers->resizeColumnToContents(0);
        vueFichiers->setColumnWidth(0, 240); //To set the column's width (here column 0).

    mkdirButton = new QPushButton(tr("&Creer un repertoire"));//To create a directory from the view.
    removeButton = new QPushButton(tr("&Supprimer"));//To remove files and directories from the view.
    boutonActualiser = new QPushButton("Actualiser");//To update the view.

    connect(mkdirButton, SIGNAL(clicked()), this, SLOT(createDirectory()));
    connect(removeButton, SIGNAL(clicked()), this, SLOT(removeFilesOrDirectory()));
    connect(boutonActualiser, SIGNAL(clicked()), modeleFichiers, SLOT(refresh()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;
        buttonLayout->addWidget(mkdirButton);
        buttonLayout->addWidget(removeButton);
        buttonLayout->addWidget(boutonActualiser);
        buttonLayout->addStretch();

    QVBoxLayout *mainLayout = new QVBoxLayout;
        mainLayout->addWidget(vueFichiers);
        mainLayout->addLayout(buttonLayout);
        setLayout(mainLayout);

    
}

/****************************************** Implementation of the createDirectory method. (SLOT) *****************************************/
void FilesViewer::createDirectory()
{
    QModelIndex index = vueFichiers->currentIndex();
    if (!index.isValid())
        return;

    QString dirName = QInputDialog::getText(this, tr("Creer un repertoire"), tr("Nom du repertoire"));
    //QString dirName2 = QInputDialog::getText(this, tr("Creer un repertoire"), tr("Nom du repertoire"));
    if (!dirName.isEmpty()) {
        if (!modeleFichiers->mkdir(index, dirName).isValid())
            QMessageBox::information(this, tr("Creer un repertoire"),
                    tr("Impossible de creer le repertoire"));
    }
}

/***************************************** Implementation of the removeFilesOrDirectory ******************************************************/
void FilesViewer::removeFilesOrDirectory()
{
    QModelIndex index = vueFichiers->currentIndex();
    if (!index.isValid())
        return;

    bool ok;
    if (modeleFichiers->fileInfo(index).isDir()) {
        ok = modeleFichiers->rmdir(index);
    } else {
        ok = modeleFichiers->remove(index);
    }
    if (!ok)
        QMessageBox::information(this, tr("Supprimer"),
                tr("Impossible de supprimer %1").arg(modeleFichiers->fileName(index)));
}

/************************************** Implementation of the updateTheView method ***********************************************/
void FilesViewer::updateTheView()
{
    modeleFichiers->refresh();//To refresh the view.
}

/*******************************************************************************************************************************/
