//! \file    InsertFCTWindow.cpp
//! \brief   Implementation of the Defval window for visual_YAO.
//! \version 2010/10/21 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.

#include "include/InsertFCTWindow.hpp"

using namespace std;
using namespace yao;

/********************* Implementation of the contrutor InsertFCTWindow(Table <Function> *aFunctionTable, QWidget *parent) ***************************/
InsertFCTWindow::InsertFCTWindow(Table <Function> *aFunctionTable, QWidget *parent) : QWidget(parent) { /*!< The constructor. */
	setupUi(this);

	m_insertFCTModel	= new QStandardItemModel(0, 2);	
	localFunctionTable	= aFunctionTable;
	
	
	QRegExp regExp("[A-Za-z_][A-Za-z0-9_]*");
 	m_insertFCTNameLine->setValidator(new QRegExpValidator(regExp, this));

	m_insertFCTModel->setHorizontalHeaderLabels(QStringList() << "Nom Fonction" << "Argument"); /*!< displays tableWidgetfct */

	m_insertFCTTableView->setModel(m_insertFCTModel);

	m_insertFCTTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
	m_insertFCTTableView->setShowGrid(false);
	m_insertFCTTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
	m_insertFCTTableView->resizeColumnsToContents();
			
      	for (Table<Function>::iterator it = localFunctionTable->begin(); it != localFunctionTable->end(); it++) {
		m_insertFCTModel->setItem(m_insertFCTModel->rowCount(), 0, new QStandardItem(QString(it->getName().c_str())));	
		it->isParameterized()? m_insertFCTModel->setItem(m_insertFCTModel->rowCount()-1, 1, new QStandardItem(QString("oui"))) : m_insertFCTModel->setItem(m_insertFCTModel->rowCount()-1, 1, new QStandardItem(QString("non")));
	}

	
	
	connect(m_insertFCTAddButton, SIGNAL(clicked()), this, SLOT(addInsertRow())); /*!< To add rows in the tableview after clicking on the button "Add"*/
	connect(m_insertFCTDelButton, SIGNAL(clicked()), this, SLOT(deleteRowFCT())); /*!< This slot delete a row after a click on the button "Supprimer" */
	connect(m_insertFCTQuitButton, SIGNAL(clicked()), this, SLOT(close())); /*!< Will be closed after a click on the button "Quit" */  
	connect(m_insertFCTHelpButton, SIGNAL(clicked()), this, SLOT(callInsertFCTHelp()));   /*!< This slot opens */ 

	setFixedWidth(baseSize().width() + 345);
}

/************************************* Implementation of the addInsertRow method() **********************************************************/
void InsertFCTWindow::addInsertRow() {

	try	
	{
		
		int row = m_insertFCTModel->rowCount();//To have the current row of the tableView.		
		string name     = m_insertFCTNameLine->text().toStdString();
	        bool arg  = m_insertFCTAgumentCheckBox->isChecked(); //? "oui" : "non" ;
		ostringstream reg;
			reg << arg;
		for (Table<Function>::iterator it = localFunctionTable->begin(); it != localFunctionTable->end(); it++)
			if (it->getName() == name) {
				QMessageBox::critical(this, "Avertissement", "Ce nom de fonction existe déjà.");
				m_insertFCTNameLine->setText("");
				return;
			}
			
		if (!name.empty()) { 
			localFunctionTable->push_back(Function(name, arg));
			m_insertFCTModel->setItem(row, 0, new QStandardItem(QString(name.c_str())));
			reg.str() == "0" ? m_insertFCTModel->setItem(row, 1, new QStandardItem("non")) : m_insertFCTModel->setItem(row, 1, new QStandardItem("oui"));
			m_insertFCTNameLine->setText("");
			m_insertFCTAgumentCheckBox->setCheckState(Qt::Unchecked);
		}
 		else
			QMessageBox::critical(this, "Erreur", "Le champ \"Nom fonction\" ne doit pas être vide!");

	} 


	catch(...) 
	{

		QMessageBox::critical(this, "Erreur", "Erreur dans la méthode addInsertRow()");	
	}
}

/************************************ Implementation of the deleteRowFCT method() ***********************************/
void InsertFCTWindow::deleteRowFCT() {

	try	
		{
			QItemSelectionModel *selection = m_insertFCTTableView->selectionModel();
			index = selection->currentIndex();
			QVariant element = m_insertFCTModel->data(index, Qt::DisplayRole);
			int r = index.row();
			if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
				{
					m_insertFCTModel->removeRows(m_insertFCTTableView->currentIndex().row(), 1);
					localFunctionTable->erase(localFunctionTable->begin() + r);

				}

			//if (isSpaceTableEmpty()) m_spaceDeleteButton->setDisabled(true);
				
		
		} 

		catch(...) 
		{

			QMessageBox::critical(this, "Erreur", "Erreur dans la méthode deleteInsertRow()");	
		}
}

/*************************************** Implementation of callInsertFCTHelp() method *******************************/
void InsertFCTWindow::callInsertFCTHelp() {

	QMessageBox::about(this, tr("Aide sur la directive insert_fct."),("<p> Cette directive permet à Yao de prendre en compte des instructions propres de l’application de l'utilisateur. Cela doit se traduire par le développement spécifique des fonctions correspondantes. Si le paramètre arg est précisé, la fonction sera appelée avec un passage de paramètres dont les arguments seront : int argc et char *argv[] (comme un programme c). <p><u>Exemple:</u> insert_fct arg xivg"
"<p>pour plus d'informations sur la directive,  <a href=\"help/insertfct_aide.html\">cliquez ici.</a>"));

}

/****************************************************************************************/
/****************************************************************************************************************************/

/*void InsertFCTWindow::()
{

}

*/  

/***********************************************************************************************/
/*void InsertFCTWindow::() {
	
	
}*/

/***********************************************************************************************/
/*bool InsertFCTWindow::() {
	
	
}*/
/*********************************** Implementation of closeEvent(QCloseEvent *event) method **********************************/
void InsertFCTWindow::closeEvent(QCloseEvent *event) {

	emit insertFCTWindowIsClosed(11);

}

/*****************************************************************************************************************************/
/********************************************************** End of file. *****************************************************/
/*****************************************************************************************************************************/
