//! \file    InstructionWindow.cpp
//! \brief   Implementation of the instructions file's window for visual_YAO.
//! \version 2010/09/30 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.

/*
  Nom du Fichier: InstructionWindow.cpp
  Nom de classe: InstructionWindow
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

#include "include/InstructionWindow.hpp"

using namespace std;
using namespace yao;


//! \class OrderWindow(QWidget *parent) : QWidget(parent)
//! \brief Implementation of the Order window for visual_YAO.

/*********************************************************************************************************************************/
InstructionWindow::InstructionWindow(vector <string> *anInstructionData, Table <Function> *aFunctionTable, QWidget *parent) : QWidget(parent), savedData(" ") {

	setupUi(this);
	
	localFunctionTable      = aFunctionTable;	
	m_localInstructionData	= anInstructionData;	

	m_rightButton->setIcon(QIcon("Images/icons/rightarrow2.png"));
	m_instructionLeftModel = new QStandardItemModel(0, 1);//Model in one column whitch contained the possibles functions used in the process of generation of the instruction file. 
	m_instructionRightModel = new QStandardItemModel(0, 2);//Model in two columns whitch contained the selected functions used in the process of generation of the instruction file. 

	initialisation = new QStandardItem("Fonctions d'initialisation");//Declaration of the first main line.

	m_instructionLeftModel->appendRow(initialisation);//Insertion of the first main line. 
	initialisation->appendRow(new QStandardItem("set_modeltime"));//Insertion of a sub-line.
	initialisation->appendRow(new QStandardItem("setstate"));//Insertion of a sub-line.
	initialisation->appendRow(new QStandardItem("set_scoef"));//Insertion of a sub-line.
	initialisation->appendRow(new QStandardItem("set_bcoef"));//Insertion of a sub-line.
	initialisation->appendRow(new QStandardItem("set_pcoef"));//Insertion of a sub-line.

	minimisation = new QStandardItem("Fonctions de minimisation");//Declaration of the 2nde main line.

	m_instructionLeftModel->appendRow(minimisation); //Insertion of the main line.
	minimisation->appendRow(new QStandardItem("setm_impres"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_io"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_mode"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_nsim"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_dxmin"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_epsg"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_ddf1"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("set_nbiter"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_xinf"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setm_xsup"));//Insertion of a sub-line.
	minimisation->appendRow(new QStandardItem("setepsi"));//Insertion of a sub-line.

	input_output = new QStandardItem("Fonctions d'entrée/sortie");

		m_instructionLeftModel->appendRow(input_output);
		input_output->appendRow(new QStandardItem("loadobs"));
		input_output->appendRow(new QStandardItem("loadstate"));
		input_output->appendRow(new QStandardItem("outoobs"));
		input_output->appendRow(new QStandardItem("outocbx"));
		input_output->appendRow(new QStandardItem("savestate"));
		input_output->appendRow(new QStandardItem("print_time"));
		input_output->appendRow(new QStandardItem("print_cost"));

	test = new QStandardItem("Fonctions de test");
		m_instructionLeftModel->appendRow(test);
		test->appendRow(new QStandardItem("testdf"));
		test->appendRow(new QStandardItem("testad"));

	execution = new QStandardItem("Fonctions d'exécution");
		m_instructionLeftModel->appendRow(execution);
		execution->appendRow(new QStandardItem("forward"));
		execution->appendRow(new QStandardItem("backward"));
		execution->appendRow(new QStandardItem("run"));
		execution->appendRow(new QStandardItem("runm"));
		execution->appendRow(new QStandardItem("runm2"));
		execution->appendRow(new QStandardItem("runim"));
	
	user_functions = new QStandardItem("Fonction(s) de l'utilisateur.");
		m_instructionLeftModel->appendRow(user_functions);
		for (Table<Function>::iterator it = localFunctionTable->begin(); it != localFunctionTable->end(); it++) {
			user_functions->appendRow(new QStandardItem(QString(it->getName().c_str())));
		}


	m_instructionLeftModel->setHorizontalHeaderLabels(QStringList() << "Fonctions proposées");
	m_instructionRightModel->setHorizontalHeaderLabels(QStringList() << "Fonctions choisies" << "Paramètres");


	m_proposedParametersListView->setModel(m_instructionLeftModel);	
	m_choosedParametersTableView->setModel(m_instructionRightModel);	
	m_choosedParametersTableView->resizeColumnsToContents();
	
	connect(m_proposedParametersListView, SIGNAL(clicked(QModelIndex )), this, SLOT(selectedFunction(QModelIndex )));
	connect(m_rightButton, SIGNAL(clicked()), this, SLOT(addInstructionRow()));
}

/********************************* Implementation of the selectedFunction(QModelIndex index) method  ***********************************/
void InstructionWindow::selectedFunction(QModelIndex index) {
	
	QString tampon;

	QItemSelectionModel *selection = m_proposedParametersListView->selectionModel();
	//index = selection->currentIndex();
	QVariant element = m_instructionLeftModel->data(index, Qt::DisplayRole);
	QString chaine = element.toString();//.toStdString();

	for (int i = 0; i < 8 ; i++) {
           
                 tampon.append(chaine.at(i));// To extract the height first characteres from the string (which will be compared with "Fonction").    
                 
         }
	
	if (tampon.compare("Fonction", Qt::CaseSensitive) == 0) 
		return;
	else {
		
		m_fonctionNameLineEdit->setText(chaine);
	}
			

	//QMessageBox::about(this, "Avis", chaine);
	//int r = index.row();
	
}

/********************************* Implementation of the addInstructionRow()  method *************************************************/
void InstructionWindow::addInstructionRow() {

try {
	QString nom  = m_fonctionNameLineEdit->text();
        QString val  = m_fonctionParameterLineEdit->text();
	
	/*QString tampon;
	
	for (int i = 0; i < 8 ; i++) {
		
		tampon.append(nom.at(i));// To extract the nine first characteres from the string (which will be compared with "Fonctions").  	
				
	}*/
	if (nom.isEmpty()){ 
  
                            QMessageBox::critical(this, "Avertissement", "Le champ \"<I>Nom de la fonction<I>\" est obligatoire! Veuillez sélectionner une fonction parmi celles qui sont proposées dans la partie gauche de la fenêtre.");
                            return; // Arrêt de la méthode
                    }
  
          else {
                  //if (!(m_ligneNom->text().isEmpty()) || (m_ligneValeur->text().isEmpty())) {
                    //      m_delButton->setEnabled(true);
                    //      Constant localConstant (Nom.toStdString(), Val.toStdString());
                    //              m_localConstantTable->push_back(localConstant);
			
				m_instructionRightModel->setItem(m_instructionRightModel->rowCount(), 0, new QStandardItem(nom));
				m_instructionRightModel->setItem(m_instructionRightModel->rowCount()-1, 1, new QStandardItem(val));
				
				m_fonctionNameLineEdit->setText("");
				m_fonctionParameterLineEdit->setText("");
				
				//QString dataInOneLine = 
				
				//savedData.push_back(nom+" "+val);
				m_localInstructionData->push_back(nom.toStdString()+" "+val.toStdString());//=  (QStringList() << str1 << str2 << str3);
				m_rightButton;	
				emit activeTheIAction();
                  }
	}	

	catch (...) {

		QMessageBox::critical(this, "Erreur", "Erreur dans la méthode addInstructionRow()");
	}
	

}
/************************************ Implementation of getInstruction() method ************************************************************/
QStringList InstructionWindow::getInstruction() {

	return savedData; 

}
/***********************************************************************************************************************/
/***************************************************** End of file *****************************************************/
/***********************************************************************************************************************/
