//! \file    OrderWindowController.cpp
//! \brief   Implementation of the Order window for visual_YAO.
//! \version 2010/10/29 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane & MAYOMBO Alexis.

/*
  Nom du Fichier: OrderWindowController.cpp
  Nom de classe: OrderWindowController
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

#include <iostream>
#include "include/OrderWindowController.hpp"
//! \class OrderWindowController(QWidget *parent): QWidget(parent);
//! \brief Implementation of the Order window for visual_YAO.

using namespace std;
using namespace yao;

/*************************************************************** The constructor **********************************************************************/
OrderWindowController::OrderWindowController(Table <Order> *aOrderTable, vector <string> *aHeaderList, MainWindowController *mwc, QWidget *parent): QWidget(parent), m_localOrderTable(aOrderTable), localHeaderList(aHeaderList), localMwc(mwc) { /*!< The constructor. */
	setupUi(this);	
	//m_localOrderTable 	= aOrderTable;
	//localHeaderList		= aHeaderList;
	//localMwc		= mwc;
	//localOrderController = aOrderController;
	QRegExp regExp("[A-Za-z_][A-Za-z0-9_]*"); //{0,}"); * means zero or more occurrence of the 2nd bloc. 
	QRegExp regExp2("[A-Za-z0-9_][A-Za-z0-9]*[.0-9]?[0-9]*[eE]?[+-]?[0-9]+");
	m_ligneNom->setValidator(new QRegExpValidator(regExp, this));		
	m_ligneValeur->setValidator(new QRegExpValidator(regExp2, this));

	if(isOrderTableEmpty()) m_delButton ->setDisabled(true);	  /*!< setDisabled is true. */

	m_defvalModel = new  QStandardItemModel(0, 2, this);//Nombre de lignes et colonnes du modèle.
		m_defvalModel->setHorizontalHeaderLabels(QStringList() << "Ordere(s)" << "Valeur(s)"); /*!< displays the QTableWidget */

	for (Table<Order>::iterator it = m_localOrderTable->begin(); it != m_localOrderTable->end(); it++)
		{
			m_defvalModel->setItem(m_defvalModel->rowCount(), 0, new QStandardItem(QString(it->getName().c_str())));
			m_defvalModel->setItem(m_defvalModel->rowCount()-1, 1, new QStandardItem(QString(it->getText().c_str())));//Apres le 1er ajout, la ligne vaut i+1
		}
	//if (mwc->isStringLiteral("malkj")) cout << "cest bon ça" << endl; else cout << "Oh dommage!" << endl;
	m_tableView->setModel(m_defvalModel);
	m_tableView->setEditTriggers(QAbstractItemView::NoEditTriggers); /*!< this ligne give no edit mode*/
	m_tableView->setShowGrid(false);  /*!< this ligne cover the grille in table */
	m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);  /*!< this ligne give a section of the ligne */
	m_tableView->resizeColumnsToContents();


	connect(m_helpButton, SIGNAL(clicked()), this, SLOT(callOrderHelp()));
	connect(m_quitButton, SIGNAL(clicked()), this, SLOT(close()));  /*!< slot for the close this windows*/
	connect(m_addButton, SIGNAL(clicked()), this, SLOT(addRowInTable()));  /*!< This slot add the object in the view */
	connect(m_delButton, SIGNAL(clicked()), this, SLOT(deleteRow())); /*!< To delete the object selected in the view*/
	//connect(this, SIGNAL(dataAdded()), this, SLOT(saveData())); /*!< This slot save data in structure */	
	
	setFixedWidth(baseSize().width() + 335);
	
	//	move(0, 0);
}

/*******************************************************************************************************************************************************/
/****************************************************************** addRowInTable() method *************************************************************/
void OrderWindowController::addRowInTable() {
try {  
	QString Nom  = m_ligneNom->text();
	QString Val  = m_ligneValeur->text();
	
	if (!m_localOrderTable->empty())
	for (Table<Order>::iterator it = m_localOrderTable->begin(); it != m_localOrderTable->end(); it++)
				if (it->getName() == Nom.toStdString())
					{
						QMessageBox::critical(this, "Avertissement", "Ce nom de Ordere existe déjà!");
						eraseDataFromLineEdit();
						return;
					}

	if ((m_ligneNom->text().isEmpty()) || (m_ligneValeur->text().isEmpty())) {

                          QMessageBox::critical(this, "Avertissement", "Avant de cliquer sur le bouton \"Ajouter\", assurez-vous de n'avoir laissé aucun champ vide.");
                          return; // Arrêt de la méthode
                  }
  
	else {
	 	if (!(m_ligneNom->text().isEmpty()) || (m_ligneValeur->text().isEmpty())) {
	
						m_delButton->setEnabled(true);                     

			Order *c; 

			c = m_localOrderTable->find(Val.toStdString()); 	
			if (c != NULL)
				Val	= QString(c->getText().c_str());//QString(m_localOrderTable->getText().c_str());
			else {
				
				for(int i=0; i < Val.size(); i++)
					if (!(Val[i] >= '0' && Val[i] <= '9')) {
	
						QMessageBox::critical(this, "Avertissement", "Seuls les noms des Orderes defval sont acceptés sous forme de chaines.");
						return;
					}
					//else
			}

			Order localOrder (Nom.toStdString(), Val.toStdString());	
			//for (Table<Order>::iterator it = m_localOrderTable->begin(); it != m_localOrderTable->end(); it++)
			//	if (m_localOrderTable->size() == 0 )// it->getName().c_str() == "" /*Nom.toStdString()*/)
					
			//		{	
						//cout << it->getName() << endl;
				
			//	else {
			//		QMessageBox::critical(this, "Avertissement", "Cet élément existe déjà!");
			//		return; // Arrêt de la méthode
			//		}
		
			m_localOrderTable->push_back(localOrder);
			m_defvalModel->setItem(m_defvalModel->rowCount(), 0, new QStandardItem(Nom));
			m_defvalModel->setItem(m_defvalModel->rowCount()-1, 1, new QStandardItem(Val));//Apres le 1er ajout, la ligne vaut i+1
				
			emit rowIsAdded(); // signal emis lorsqu'une ligne a été correctement rajoutée dans le tableau. 
					   // Il sera recupéré dans le mainwidow.
		}
	}

	eraseDataFromLineEdit();
  }
  catch (...) {
	
	QMessageBox::critical(this, "Erreur", "Erreur dans la méthode addDefvalRow()");
	}
 }

/*******************************************************************************************************************************************************/
/******************************************************************* delete Row() method ***************************************************************/
void OrderWindowController::deleteRow() {
try {
	string OrderNameCopy, OrderValueCopie;

	//int c = m_tableView->currentIndex().row();//renvoie le numero de la ligne selectionnée.
	QItemSelectionModel *selection = m_tableView->selectionModel();
	index = selection->currentIndex();
	QVariant element = m_defvalModel->data(index, Qt::DisplayRole);
	int r = index.row();
	if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
         {
		m_defvalModel->removeRows(m_tableView->currentIndex().row(), 1);
		
		emit rowIsDeleted(element.toString()); // signal emis lorsqu'une ligne est retirée du tableau. 
		
		m_localOrderTable->erase(m_localOrderTable->begin() + r);



		string chaine = element.toString().toStdString();
		for (Table<Order>::iterator it = m_localOrderTable->begin(); it != m_localOrderTable->end(); it++)
			{
				
				//cout << "Chaine dans la boucle: " << chaine << endl;
				if (chaine == it->getName())
				chaine = it->getText().c_str();
			}
				//cout << "Chaine après la boucle: " << chaine << endl;
	     for (int i=0; i < localHeaderList->size(); i++ ) //Parcourt le vecteur de HatName en fonction de sa taille (nombre d'éléments)
		{
			
			//localOrderTable->find(hatFile)->getText();	
			//Order *OrderForUptime;
			//chaine = m_localOrderTable->find(chaine)->getText();
			//element.toString().toStdString();
			if (element.toString().toStdString() == localHeaderList->at(i)) //Teste si l'élément à supprimer dans defval est aussi dans hat
		     		localHeaderList->erase(localHeaderList->begin() + i);//localHeaderList->begin() +i );	
		}





		/*string chaine = element.toString().toStdString();
		for (Table<Order>::iterator it = m_localOrderTable->begin(); it != m_localOrderTable->end(); it++)
			{
				
				//cout << "Chaine dans la boucle: " << chaine << endl;
				if (chaine == it->getName())
				chaine = it->getText().c_str();
			}
				//cout << "Chaine après la boucle: " << chaine << endl;
	     //for (unsigned int i=0; i < localHeaderList->size(); i++ ) //Parcourt le vecteur de HatName en fonction de sa taille (nombre d'éléments)
	//	{
			
			
	
			//localOrderTable->find(hatFile)->getText();	
			//Order *OrderForUptime;
			//chaine = m_localOrderTable->find(chaine)->getText();
			//element.toString().toStdString();
		//	if (element.toString().toStdString() == localHeaderList->at(i)) //Teste si l'élément à supprimer dans defval est aussi dans hat
		//		localHeaderList->erase(localHeaderList->begin() + i);//localHeaderList->begin() +i );	
	//	}
		//ostringstream t << element.toString().toStdString();
		//string test = m_localOrderTable->find(element.toString().toStdString())->getText();	
		
		//emit rowIsDeleted(element.toString()); // signal emis lorsqu'une ligne est retirée du tableau. 
		//cout << "voici le mot supprimé:" << element.toString().toStdString() << endl;
		*/
		if(isOrderTableEmpty()) m_delButton->setDisabled(true);
	}
   }
   catch(...) {

	QMessageBox::critical(this, "Erreur", "erreur dans la suppression d'une ligne dans deleteDefvalRow().");
	}	
}
 
/*******************************************************************************************************************************************************/
/*************************************************** eraseDataFromLineEdit() ***************************************************************************/
void OrderWindowController::eraseDataFromLineEdit() {

	m_ligneNom->setText("");
	m_ligneValeur->setText("");
}
/*******************************************************************************************************************************************************/
/*******************************************************************************************************************************************************/
void OrderWindowController::saveData() {
 
         m_quitButton->setEnabled(true);
	
}

/*******************************************************************************************************************************************************/
bool OrderWindowController::isOrderTableEmpty() {

	if (m_localOrderTable->empty())
		return true;
	else
		return false;
}
/******************************************************** callOrderHelp() ***************************************************************************/
void OrderWindowController::callOrderHelp() {

	QMessageBox::about(this, tr("Aide sur la directive Defval"),
			("<p> La directive Defval appelée aussi Order permet d'associer un nom symbolique à une valeur. Voici les différents emplois:"
                         "<p><u>Exemple 1</u>: defval szx 12 (nombre entier douze) <p><u>Exemple 2</u>: defval szy \"chaine\" (chaine de caractères)<p> <u>Exemple 3</u>: defval szz szx (nom d'une autre Ordere defval)."
			 "<p>Dans la fenêtre graphique, vous devez renseigner les champs nom et valeur de la Ordere. Visual Yao vérifie la syntaxe " 
			 "et vous guide au fur et à mesure." 
			 "<br>Le bouton \"Ajouter\" permet d'insérer des nouveaux enregistrements (ici le nom et la valeur d'une Ordere) dans la structure de données); "
			 "<br>Le bouton \"Supprimer\" permet de retirer un enregistrement sélectionné de la structure de données;"
			 "<br> Le bouton \"Quitter\" permet de fermer la fenêtre;" 
			 "<br> Le bouton \"Aide\" permet d'afficher cette fenêtre."
                         "<p> Pour plus d'informations sur defval, <a href=\"help/Ordere_aide.html\">cliquez ici.</a>"));

}
/*******************************************************************************************************************************************************/
/*******************************************************************************************************************************************************/
void OrderWindowController::closeEvent(QCloseEvent *event) {
	
	if (!(m_ligneNom->text().isEmpty()) && !(m_ligneValeur->text().isEmpty())) { //When the two QLineEdit content data, show a confirmation box.	
	
	int ret = QMessageBox::warning(this, tr("Confirmation de fermeture"), tr("Voulez-vous quitter sans sauvegarder les données contenues dans les champs?"),
                          QMessageBox::Yes | QMessageBox::Default, QMessageBox::Cancel | QMessageBox::Escape);
	
	if (ret == QMessageBox::Yes) 
                 event->accept();
               else
                 event->ignore();

	}
        	emit windowIsClosed(1); //The signal is emitted with parameter "1" which is used for "DevalWindow". 	

	//	~OrderWindowController();
}

/*******************************************************************************************************************************************************/
/************************************************ Le destructeur ~OrderWindowController() ********************************************************************/
OrderWindowController::~OrderWindowController() {

/*	//delete m_localOrder;
        //delete localHeaderList;
	delete m_localOrderTable;
	delete m_localOrder;
	delete localHeaderList;
 
	delete m_modeleListeNoms;
	delete m_defvalModel;
	delete localMwc;
*/
	

 
}
/****************************************************************************************************************************************************/
/************************************************** The end of the file. ****************************************************************************/
/****************************************************************************************************************************************************/
