//! \file    ModulWindow.cpp
//! \brief   Implementation of the Modul window for visual_YAO.
//! \version 2010/10/21 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.

/*
  Nom du Fichier: ModulWindow.cpp
  Nom de classe:
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

#include "ModulWindow.hpp"

using namespace std;
using namespace yao;

//! \class ModulWindow(QWidget *parent) : QWidget(parent)
//! \brief Implementation of the Modul window for visual_YAO.
//ModulWindow::ModulWindow(QWidget *parent):QWidget(parent) {
ModulWindow::ModulWindow(Table <Modul> *aModulTable, Table <Space> *aSpaceTable, Table <Operator> *anOperatorTable, Table <Constant> *aConstantTable, MainWindowController *mwc, string *aProjectName, string *aProjectPath, QWidget *parent) : QWidget(parent), m_localModulTable(NULL)/*, Modul(m_localModulName)*/ {


	setupUi(this);
	localConstantTable	= aConstantTable;
	m_localModulTable	= aModulTable;
	m_localSpaceTable	= aSpaceTable;
	m_localOperaTable	= anOperatorTable;
	m_localProjectName	= aProjectName;
	m_localProjectPath	= aProjectPath;
	

	QRegExp regExp("[A-Za-z_][A-Za-z1-9_]{0,}"); 
	m_modulNameLine->setValidator(new QRegExpValidator(regExp, this));
	QRegExp regExp2("[1-9]{0,}"); 
	m_modulInputLine->setValidator(new QRegExpValidator(regExp2, this));
	m_modulOutputLine->setValidator(new QRegExpValidator(regExp2, this));
	
	//Modul(&m_localModulName);

	m_modulOperaCombo->setDisabled(true);
	//m_modulTargetCombo->insertItem(0, " ");
	//m_modulTargetCombo->insertItem(1, "uptime");
	//m_modulTargetCombo->insertItem(2, "steptime");
	//m_modulTargetCombo->insertItem(3, "alltime");

	m_modulInter2Combo->setDisabled(true);
	m_modulInter3Combo->setDisabled(true);
	
	//cout << "voici le chemin du projet courant: " << endl << *m_localProjectPath + "/" + *m_localProjectName << endl;
	
	if (isModulTableEmpty()) 
		m_modulDelButton->setDisabled(true);	
	//m_modulArrayCheckBox->setDisabled(true);
//	for (Table<Constant>::reverse_iterator it = localConstantTable->rbegin(); it != localConstantTable->rend(); it++)  {
//				cout << it->getName().c_str() << endl;
		//m_modulSpaceComboName->insertItem(2, "DOUBLE"); 
//		m_modulAutonetCombo->insertItem(2, QString(it->getName().c_str()));
		//m_comboModulClonof->insertItem(1, QString(it->().c_str()));
//		m_comboModulLopera->insertItem(1, QString(it->getName().c_str())); 

	m_modulSpaceCombo->insertItem(0, " ");
	m_modulOperaCombo->insertItem(0, " ");

	for (Table<Space>::reverse_iterator it = m_localSpaceTable->rbegin(); it != m_localSpaceTable->rend(); it++)  
		m_modulSpaceCombo->insertItem(1, QString(it->getProperty(0).c_str()));
		
	for (Table<Space>::reverse_iterator it = m_localOperaTable->rbegin(); it != m_localOperaTable->rend(); it++)  
		m_modulOperaCombo->insertItem(1, QString(it->getProperty(0).c_str()));
		
	m_modulModel 	= new QStandardItemModel(0, 12);
        	m_modulModel->setHorizontalHeaderLabels(QStringList() << "Nom module" << "space/opera" << "input" << "output" << "target" << "noward" << "autonet" << "netward" << "spec" << "cout" << "lopera" << "tempo");
			
		m_modulTableView->setModel(m_modulModel);
		m_modulTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
		m_modulTableView->setShowGrid(false);
		m_modulTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
		m_modulTableView->resizeColumnsToContents();

	for (Table<Modul>::iterator it = m_localModulTable->begin(); it != m_localModulTable->end(); it++) {
 
             int row = m_modulModel->rowCount();
 
             m_modulModel->setItem(row, 0, new QStandardItem(QString(it->getName().c_str()))); /*!< Gets the name.*/
             m_modulModel->setItem(row, 1, new QStandardItem(QString(it->getSpaceOrOperator().c_str()))); /*!< Gets the dimension (1, 2 or 3).*/
		if (it->isInput())
			m_modulModel->setItem(row, 2, new QStandardItem(QString(intToString(it->getInput()).c_str()))); /*!< Gets the  choosed in the space directive .*/
		if (it->isOutput())
			m_modulModel->setItem(row, 3, new QStandardItem(QString(intToString(it->getOutput()).c_str()))); /*!< Gets the  choosed in the space directive .*/
		//if (it->isTarget()) 
		//	m_modulModel->setItem(row, 4, new QStandardItem(m_modulTargetCombo->itemText(it->getTarget()-1))); 
           }


	if (m_modulInputLine->text().isEmpty())
		m_modulArrayCheckBox->setDisabled(true);
	//connect(m_modulInputLine, SIGNAL(textEdited(QString )), this, SLOT(close()));
	
	connect(m_modulQuitButton, SIGNAL(clicked()), this, SLOT(close()));
        connect(m_modulAddButton, SIGNAL(clicked()), this, SLOT(addModulRow()));
	connect(m_modulDelButton, SIGNAL(clicked()), this, SLOT(deleteModulRow()));
	connect(m_modulHelpButton, SIGNAL(clicked()), this, SLOT(callModulHelp()));
	
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_modulSpecCheckBox, SLOT(setDisabled(bool )));
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_modulNetwardCheckBox, SLOT(setDisabled(bool )));
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_labelModulAutonet, SLOT(setDisabled(bool )));
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_modulAutonetCombo, SLOT(setDisabled(bool )));

	connect(m_modulSpaceRadioButton, SIGNAL(toggled(bool )), m_modulOperaCombo, SLOT(setDisabled(bool )));
	connect(m_modulOperaRadioButton, SIGNAL(toggled(bool )), m_modulSpaceCombo, SLOT(setDisabled(bool )));

        connect(m_modulCoutCheckBox, SIGNAL(toggled(bool )), m_modulNowardCheckBox, SLOT(setChecked(bool )));

	connect(m_modulTempoCheckBox, SIGNAL(toggled(bool )), m_modulCovtCombo, SLOT(setDisabled(bool )));
	//connect(m_modulTempoCheckBox, SIGNAL(toggled(bool )), , SLOT(setDisabled(bool )));
	
        connect(m_buttonLoperaAdd, SIGNAL(clicked()), this, SLOT(addLoperaRow()));
	connect(m_buttonLoperaDel, SIGNAL(clicked()), this, SLOT(deleteLoperaRow()));

	setFixedWidth(baseSize().width() + 780);

}

/**************************************** Implementation of addModulRow method *****************************************************************/
void ModulWindow::addModulRow() {
try {
		string 	modulName 	= m_modulNameLine->text().toStdString() ,
		modulParent		= "",
		input		= m_modulInputLine->text().toStdString(),
		output		= m_modulOutputLine->text().toStdString(),
		//target		= m_modulTargetCombo->currentText().toStdString(),
		modulAttributs;
	
	int row = m_modulModel->rowCount();

	//modulname	
	if (modulName.empty())
			{
				QMessageBox::critical(this, "Erreur", "Veuillez d'abord renseigner le champ \"nom module\".");
				return; // Arrêt de la méthode
			}
	for (Table<Modul>::iterator it = m_localModulTable->begin(); it != m_localModulTable->end(); it++)
		if (it->getName() == modulName) {
			QMessageBox::critical(this, "Avertissement", "Ce nom de module existe déjà.");
			toInitialize();
			return;
		}

	
	if ((m_modulSpaceRadioButton->isChecked()) && (!m_modulSpaceCombo->currentText().isEmpty()))
		modulParent = m_modulSpaceCombo->currentText().toStdString() ;
	else
		if ((m_modulOperaRadioButton->isChecked()) && (!m_modulOperaCombo->currentText().isEmpty()))
			modulParent = m_modulOperaCombo->currentText().toStdString() ;
	//input attribut.	
	if(!input.empty()) 
		{
			 
			modulAttributs = "input " + input;
			if(!output.empty())
				modulAttributs += "output " + output;
		}	
	              
	//output attribut.	
	if (!(output.empty() && input.empty()) && modulName.empty())
			{
				QMessageBox::critical(this, "Erreur", "Veuillez d'abord renseigner le champ \"nom module\".");
				return; // Arrêt de la méthode

			}
	// Target attribut.
	/*if(!target.empty())
                 {
                         modulAttributs = "target " + target;
                 }*/ 
	// the modul parent must to be here.
	if (modulParent.empty()) 
		{
			QMessageBox::critical(this, "Erreur", "Veuillez renseigner la valeur du champ \"space or opera\".");
                        return; // Arrêt de la méthode

		}
	if (m_modulTempoCheckBox->isChecked() && output.empty())
		{
			QMessageBox::critical(this, "Erreur", "Si target est coché, output doit-être obligatoirement renseigné.");
                        return; // Arrêt de la méthode

		}
	//if (!(!target.empty() && output.empty())) {
	//			QMessageBox::critical(this, "Erreur", "Si target est défini, output doit-être obligatoirement renseigné.");
	//			return; // Arrêt de la méthode
	//		}

	if (!modulName.empty() && !modulParent.empty()) 
		{ // !(false) => true if there are data in the lineEDit.
	
			m_modulModel->setItem(row, 0, new QStandardItem(QString(modulName.c_str())));  
			m_modulModel->setItem(row, 1, new QStandardItem(QString(modulParent.c_str())));  
			if (!input.empty())
				{ 
					m_modulModel->setItem(row, 2, new QStandardItem(QString(input.c_str())));  
					if (!output.empty()) 
						m_modulModel->setItem(row, 3, new QStandardItem(QString(output.c_str())));
				}
			if (!(output.empty()) && (input.empty())) {
				QMessageBox::critical(this, "Erreur", "Veuillez préciser la valeur du champ \"input\".");
				return; // Arrêt de la méthode
			} 
			//if (!target.empty() && (!output.empty()))
			//	m_modulModel->setItem(row, 4, new QStandardItem(QString(target.c_str())));  
			
		if(m_modulNowardCheckBox->isChecked())
			{
				m_modulModel->setItem(row, 5, new QStandardItem("oui"));
				m_modulModel->setItem(row, 6, new QStandardItem("non"));
				m_modulModel->setItem(row, 7, new QStandardItem("non"));
				m_modulModel->setItem(row, 8, new QStandardItem("non"));
				
			}
		else
			{
				QMessageBox::about(this, "Avis", "Le fichier "+ m_modulNameLine->text()+".h est crée.");
                                 //return; // Arrêt de la méthode

			}
		//bool aBool = m_modulSpaceRadioButton->isChecked()? true: false;

	/*Modul mod =*/ 
			if (m_modulSpaceRadioButton->isChecked()) 
				{
					Modul mod1(modulName, m_localSpaceTable->find(modulParent), true);
					if (!input.empty())
						mod1.setInput(stringToInt(input));
					if (!output.empty())
						mod1.setOutput(stringToInt(output));
					//if (!target.empty())
						//mod1.setTarget(intToShort(m_modulTargetCombo->currentIndex()) + 1);
					m_localModulTable->push_back(mod1);

				}
			else
				{
					Modul mod2(modulName, m_localOperaTable->find(modulParent), false);	
					if (!input.empty())
						mod2.setInput(stringToInt(input));
					if (!output.empty())
						mod2.setOutput(stringToInt(output));
					//if (!target.empty())
						//mod2.setTarget(intToShort(m_modulTargetCombo->currentIndex()) + 1);
					m_localModulTable->push_back(mod2);

				}

	
		if (!m_modulNowardCheckBox->isChecked()) generateModulFile(modulName); 		
	}
	else 
		{
			QMessageBox::critical(this, "Erreur", "Veuillez renseigner le champ \"nom module\" ou \"space or opera\".");
                                 return; // Arrêt de la méthode

		}
	toInitialize();
	if (!isModulTableEmpty()) {
		 m_modulDelButton->setEnabled(true);	
		 emit modulRowIsAdded(true);
	}
  }
	catch (...) {
		
		cout << "Erreur dans la methode addModulRow" << endl;
	}   	
	toInitialize();
}

/**************************************** Implementation of addLoperaRow() method **************************************************************/
void ModulWindow::addLoperaRow() {

	m_loperaModel->appendRow(new QStandardItem(m_modulLoperaCombo->currentText())); 
	m_modulLoperaCombo->setCurrentIndex(0);
}

/**************************************** Implementation of deleteLoperaRow() method **********************************************************/
void ModulWindow::deleteLoperaRow() {

	QItemSelectionModel *selection = m_loperaListView->selectionModel(); 
	m_loperaIndex = selection->currentIndex(); 
	 if (!m_loperaIndex.isValid()) 
		return; 
	QVariant element = m_loperaModel->data(m_loperaIndex, Qt::DisplayRole);
	int r = m_loperaIndex.row();
	if (selection->hasSelection()) {
	 m_loperaModel->removeRows(m_loperaListView->currentIndex().row(), 1); 
	//localHeaderList->erase(localHeaderList->begin() + r);
	//if(isHeaderListEmpty()) m_delHatButton->setDisabled(true);
	}
}

/************************************* Implementation of toInitialize() method ***********************************************************/
void ModulWindow::toInitialize() {

	m_modulNameLine->setText("");
	m_modulInputLine->setText("");
	m_modulOutputLine->setText("");
	//m_modulTargetCombo->setCurrentIndex(0);
	m_modulSpaceCombo->setCurrentIndex(0); 
	m_modulOperaCombo->setCurrentIndex(0); 
	m_modulAutonetCombo->setCurrentIndex(0);
	m_modulClonofCombo->setCurrentIndex(0); 
	m_modulLoperaCombo->setCurrentIndex(0);
	m_modulNowardCheckBox->setChecked(false);	
	m_modulTempoCheckBox->setChecked(false);
	m_modulCoutCheckBox->setChecked(false);
	m_modulSpecCheckBox->setChecked(false);
	m_modulNetwardCheckBox->setChecked(false);
	m_modulHidjacCheckBox->setChecked(false);
	m_modulManageCheckBox->setChecked(false);
	m_modulArrayCheckBox->setChecked(false);
		
}

/*************************************** Implementation of deleteModulRow() method. ***************************************************************/
void ModulWindow::deleteModulRow() {
	try 
		{

			QItemSelectionModel *selection = m_modulTableView->selectionModel();
			index = selection->currentIndex();
			QVariant element = m_modulModel->data(index, Qt::DisplayRole);
			int r = index.row();
			if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
				 {   
					m_modulModel->removeRows(m_modulTableView->currentIndex().row(), 1); 
					m_localModulTable->erase(m_localModulTable->begin() + r); 
    
					if(isModulTableEmpty()) m_modulDelButton->setDisabled(true);
				}   
		}   

	catch(...) 
		{

			QMessageBox::critical(this, "Erreur", "erreur dans la suppression d'une ligne dans deleteModulRow().");
		}    

    //	QVariant elementSelectionne = filesListModele->data(indexElementSelectionne, Qt::DisplayRole);

     	//filesList.removeOne(elementSelectionne.toString());//Retrait de l'élément sélectionné de l'objet QStringList
     	//filesListModele->setStringList(filesList); //On indique au modèle la liste des éléments à afficher.
	
}

/*************************************** Implementation of saveModulData() method. **************************************************/
void ModulWindow::saveModulData() {
	
}
/************************************* Implementation of isModulTableEmpty() method. *********************************/
bool ModulWindow::isModulTableEmpty() {

	if (m_localModulTable->empty()) 
		return true; 
	else                                                              
		return false;   
	
}
/**************************************** Implementation of intToString(int anInt) method. ********************************************************/
string ModulWindow::intToString(int anInt) {

	stringstream tampon;
	tampon	<< anInt;
	
	return tampon.str();
}
/**************************************** Implementation of stringToInt(const string& s) method. **************************************************/
const int ModulWindow::stringToInt(const string& s) {

      return atoi(s.c_str());
}

/**************************************** Implementation of stringToShort(const string& s) method. **************************************************/
const short ModulWindow::stringToShort(const string& s) {

      return atoi(s.c_str());
}
/**************************************** Implementation of intToShort(const int anInt) method. **************************************************/
const short ModulWindow::intToShort(const int anInt) {
	
      return atoi(intToString(anInt).c_str());
}

/*************************************** Implementation of method. *******************************************************************************/
/*QStringList ModulWindow::getModulData() {

	//return filesList;
}
*/
/*************************************** Implementation of callModulHelp() *************************************************************************/
void ModulWindow::callModulHelp() {

	 QMessageBox::about(this, tr("Aide sur la directive Module."),("<p> Cette directive permet de déclarer les modules. Un module est identifié par son nom (modulname). Il doit ensuite être rattaché soit à un espace (introduit par le mot clé space suivi du nom de l’espace(spacename)) soit à un opérateur (introduit par le mot clé opera suivi du nom de l’opérateur (operaname)). <p> <u>Exemple:</u> modul Hfil space Soce input 3 output 1 tempo cout target"
                              "<p>Pour plus d'informations sur la directive module,  <a href=\"help/modul_aide.html\">cliquez ici.</a>"));

}

/************************************ Implementation of generateModulFile(string aFilename) method. *************************************************/
void ModulWindow::generateModulFile(string aFilename) 
{
	date = new QDateEdit;
    		date->setDate(QDate::currentDate());
	//string theFile ;
	string path = *m_localProjectPath + "/" + *m_localProjectName + "/" + aFilename + ".h";
	ofstream theFile(path.c_str(), ios::out | ios::binary | ios::trunc);
		if(!theFile.is_open())
			throw ofstream::failure("couldn't create " + path);
	theFile << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "//		Date: " << date->date().toString().toStdString().c_str()    	   << endl; 
	theFile << "//===========================================================================" << endl;
	theFile << "forward ()									 " << endl;
	theFile << "{										 " << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << "}										 " << endl;
	theFile << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "// methodes classe : backward						 " << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "backward ()									 " << endl;
	theFile << "{										 " << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << "}										 " << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "//**************************  FIN DU MODULE   *******************************" << endl;

	theFile.close();
}

/********************************************************************/
void ModulWindow::closeEvent(QCloseEvent *event) {

	emit modulWindowIsClosed(8)	;
} 
