//! \file    NetwardWindow.cpp
//! \brief   Implementation of the Netward window for visual_YAO.
//! \version 2010/10/21 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.

//! \class NetwardWindow(QWidget *parent) : QWidget(parent)
//! \brief Implementation of the Netward window for visual_YAO.

#include "include/NetwardWindow.hpp"

using namespace std;
using namespace yao;

NetwardWindow::NetwardWindow(yao::Table <yao::Neuron> *aNeuronTable, Table <Constant> *aConstantTable, QWidget *parent) : QWidget(parent) {   /*!< The constructor. */
	setupUi(this);

	m_localConstantTable	= aConstantTable;	
	m_localNeuronTable	= aNeuronTable;

	QRegExp regExp("[A-Za-z_][A-Za-z0-9_]*"); 
	m_nameNetwardLine->setValidator(new QRegExpValidator(regExp, this));
	
	m_netwardNbiCombo->setEditable(true);
	m_netwardNbiCombo->insertItem(0, "");

	m_netwardNbsCombo->setEditable(true);
	m_netwardNbsCombo->insertItem(0, "");

	for (Table<Constant>::reverse_iterator it = m_localConstantTable->rbegin(); it != m_localConstantTable->rend(); it++) 
	{
 
		m_netwardNbiCombo->insertItem(1, QString(it->getName().c_str()));
		m_netwardNbsCombo->insertItem(1, QString(it->getName().c_str()));
	}

	m_netwardModel	= new QStandardItemModel(0, 3);
	m_netwardModel->setHorizontalHeaderLabels(QStringList() << "Nom du réseau" << "Entrees" << "Sorties");  /*!< displays the QTableWidget */

	for (Table<Neuron>::iterator it = m_localNeuronTable->begin(); it != m_localNeuronTable->end(); it++) {

		int row = m_netwardModel->rowCount();

		ostringstream in, out;
		
			in << it->getInDegree();
			out << it->getOutDegree();

		m_netwardModel->setItem(row, 0, new QStandardItem(QString(it->getName().c_str()))); /*!< Gets the space name.*/
		m_netwardModel->setItem(row, 1, new QStandardItem(QString(in.str().c_str()))); /*!< Gets the space dimension (1, 2 or 3).*/
		m_netwardModel->setItem(row, 2, new QStandardItem(QString(out.str().c_str()))); /*!< Gets the space dimension (1, 2 or 3).*/
		//m_netwardModel->setItem(row, , new QStandardItem(QString(it->getProperty(2).c_str()))); /*!< Gets the trajectory choosed in the space directive.*/
          }
	

	m_netwardTableView->setModel(m_netwardModel);
	m_netwardTableView->setEditTriggers(QAbstractItemView::NoEditTriggers); /*!< this ligne give no edit mode*/
	m_netwardTableView->setShowGrid(false); /*!< this ligne cover the grille in table */
	m_netwardTableView->setSelectionBehavior(QAbstractItemView::SelectRows);  /*!< this ligne give a section of the ligne */
	m_netwardTableView->resizeColumnsToContents();
    
        connect(m_netwardAddButton, SIGNAL(clicked()), this, SLOT(addNetwardRow()));
        connect(m_netwardDeleteButton, SIGNAL(clicked()), this, SLOT(deleteNetwardRow()));
	connect(m_netwardQuitButton, SIGNAL(clicked()), this, SLOT(close()));
        connect(m_netwardHelpButton, SIGNAL(clicked()), this, SLOT(callNetwardHelp()));
  
  setFixedWidth(baseSize().width() + 392);
}

/****************************************************************************************/
void NetwardWindow::addNetwardRow() {
 
	int row = m_netwardModel->rowCount();//To have the current row of the tableView.

        string name		= m_nameNetwardLine->text().toStdString(),
        	m_netwardNbi	= m_netwardNbiCombo->currentText().toStdString(),
        	m_netwardNbs	= m_netwardNbsCombo->currentText().toStdString(),
        	m_netwardNbi2	,
        	m_netwardNbs2	;
    
	for (Table<Neuron>::iterator it = m_localNeuronTable->begin(); it != m_localNeuronTable->end(); it++)
		if (it->getName() == name) {
			QMessageBox::critical(this, "Avertissement", "Ce nom de réseau existe déjà.");
			toInitialize();
			return;
		}

        Constant *c1; //Nous créons d'abord un pointeur de constante.
        Constant *c2; //Nous créons d'abord un pointeur de constante.
 
        c1  = m_localConstantTable->find(m_netwardNbi); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
        c2 = m_localConstantTable->find(m_netwardNbs); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
    
        if (c1 != 0) { //C'est alors une constante.

                m_netwardNbi2 = m_localConstantTable->find(m_netwardNbi)->getText();  
    
        }   
        else {

                for(unsigned int i=0; i < m_netwardNbi.size(); i++) {
			if(!(m_netwardNbi[i] >= '0' && m_netwardNbi[i] <= '9')){
 
				QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"nbi\"."));
				return;
			}   

		}   

                m_netwardNbi2 = m_netwardNbi;    
        }   

        if (c2 != NULL) { //C'est alors une constante.

                m_netwardNbs2  =  m_localConstantTable->find(m_netwardNbs)->getText();    
    
        }
        else {
		                for(unsigned int i=0; i < m_netwardNbs.size(); i++) {
                if(!(m_netwardNbs[i] >= '0' && m_netwardNbs[i] <= '9')){

                     QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"nbs\"."));
                      return;
                 }

                }

               m_netwardNbs2 = m_netwardNbs;

        }

        if (name.empty()) {
                QMessageBox::about(this, tr("Erreur de saisie!"),
                     tr("<p>Le champ \"Nom reseau\" ne doit pas etre vide."));
                return;
        }
        else

         /*if (!.empty())*/ {
		 if (!(m_netwardNbi2.empty() || m_netwardNbs2.empty())) 
			{ 
				m_netwardModel->setItem(row, 0, new QStandardItem(QString(name.c_str())));
				m_netwardModel->setItem(row, 1, new QStandardItem(QString(m_netwardNbi2.c_str())));
		 	//	if (!m_netwardNbs2.empty()) 
				m_netwardModel->setItem(row, 2, new QStandardItem(QString(m_netwardNbs2.c_str()))); 
				int in 	= toInt(m_netwardNbi2);
				int out	= toInt(m_netwardNbs2);
				m_localNeuronTable->push_back(Neuron(name, in, out));
			}	
		else
			{
				if (m_netwardNbi2.empty())
					{
						QMessageBox::about(this, tr("Erreur de saisie!"), tr("<p>Le champ \"nbi\" ne doit pas etre vide."));
						return;

					}

				if (m_netwardNbs2.empty())
					{
						QMessageBox::about(this, tr("Erreur de saisie!"), tr("<p>Le champ \"nbs\" ne doit pas etre vide."));
						return;

					}

			}
        //if (!isSpaceTableEmpty()) m_spaceDeleteButton->setEnabled(true);
		
        }
	
	toInitialize();
}

/****************************************************************************************/
void NetwardWindow::deleteNetwardRow() {

//	m_netwardTableView->removeRow(m_netwardTableView->currentRow());
//    	isCalled = true;

}

/****************************************************************************************/
void NetwardWindow::toInitialize() {

	m_nameNetwardLine->setText("");
	m_netwardNbiCombo->setCurrentIndex(0);
	m_netwardNbsCombo->setCurrentIndex(0);

}

/****************************************************************************************/
const int NetwardWindow::toInt(const string& s) {
 
      return atoi(s.c_str());
 
}

/****************************************************************************************/
/*void NetwardWindow::ajouterNameDansTableau(int ligne, int colonne, QString nomfct1) {
    QTableWidgetItem *chaine1 = new QTableWidgetItem;
    chaine1->setText(nomfct1);

      nomfct1->setItem(ligne-1, colonne-1, new QTableWidgetItem(nomfct1));
}*/

/****************************************************************************************/
/*void NetwardWindow::ajouterInputDansTableau(int ligne, int colonne, QString entrefct1) {

QTableWidgetItem *chaine1 = new QTableWidgetItem;
chaine1->setText(entrefct1);

    entrefct1->setItem(ligne - 1, colonne, chaine1);

}*/
/****************************************************************************************/
/*void NetwardWindow::ajouterOutputDansTableau(int ligne, int colonne, QString sortifct) {
    QTableWidgetItem *chaine1 = new QTableWidgetItem;
    chaine1->setText(sortifct);

      sortifct->setItem(ligne-1, colonne-1, new QTableWidgetItem(sortifct));
}*/
/****************************************************************************************/
/*void NetwardWindow::addRowNetward() {

	int row = m_netwardTableView->rowCount();
	m_netwardTableView->insertRow(row);

}*/
/****************************************************************************************/
/*void NetwardWindow::saveNetwardData() {

	QTableWidgetItem *cel1 = new QTableWidgetItem;
	QTableWidgetItem *cel2 = new QTableWidgetItem;	
	
	for (int i = 0; i < m_netwardTableView->rowCount() ; i++) {
			cel1 = m_netwardTableView->item(i,0);
			cel2 = m_netwardTableView->item(i,1);
				
			//defvalData.push_back(cell1->text() + " " + cell2->text());
         		
		}
	//for (QStringList::iterator it = defvalData.begin(); it != defvalData.end(); it++)
	//cout << "Voici les valeurs ajoutées: " << it->toStdString() <<endl;

}*/
/****************************************************************************************/
/*void DefvalWindow::initialiserChamps() {

     	   m_ligneNomDefval->setText("");
	   m_ligneValeurDefval->setText("");
}*/

/************************************ Implémentation de la méthode callHelpDefval ****************************************/
/*void NetwardWindow::callNetwardHelp()
{
    QMessageBox::about(this, tr("Aide sur la directive Netward."),("<p> La directive Netward permet  d'associer un nom symbolique a une valeur."
                                                                  "<p><u>Exemple 1</u>: deval szx 12 <p><u>Exemple 2</u>: defval szy \"chaine\"<p> <u>Exemple 3</u>: defval szz szx"));
}*/

/****************************************************************************************/
/*QStringList NetwardWindow::getDataNetward() {

    
     return netwardData;
}*/
/**************************************************************************************/
void NetwardWindow::callNetwardHelp() {

  QMessageBox::about(this, tr("Aide sur la directive Netward."),("<p> Cette directive permet de créer le code nécessaire à une implémentation aisée, dans des modules, des fonctions réseaux de neurones.<p> Pour plus d'informations sur la directive, <a href=\"help/netward_aide.html\">cliquez ici.</a>"));

}
/**********************************************************************/
void NetwardWindow::closeEvent(QCloseEvent *event) {

	emit netwardWindowIsClosed(7);
}

