//! \file    OperaWindow.cpp
//! \brief   Implementation of the Operator window.
//! \version 2010/10/21 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.
/*
  Nom du Fichier: OperaWindow.cpp
  Nom de classe: OperaWindow
  Role:
  Nombre de methodes:11
  Nombre d'attributs:8
  */

#include "include/OperaWindow.hpp"
#include "include/TrajectoryWindow.hpp"
/*!
 * \class OperaWindow
 * \brief The cpp file of the OperaWindow class for visual_YAO.
 */

using namespace std;
using namespace yao;

/******************************************* Contructeur ***************************************************/
OperaWindow::OperaWindow(Table <Operator> *aOperaTable, Table <Trajectory> *aTrajectory, Table <Constant> *aConstantTable, QWidget *parent) : QWidget(parent), localTrajectoryTable(NULL), localOperaTable(NULL) {       /*!< The constructor. */

	setupUi(this);

	localOperaTable				= aOperaTable;  /*!< See theOperaTable in MainWindow.hpp */
	localTrajectoryTable			= aTrajectory;  /*!< See theTrajectoryTable in MainWindow.hpp */
	localConstantTable			= aConstantTable; /*!< See theConstantTable in MainWindow.hpp */
	
	QRegExp regExp("[A-Za-z_][A-Za-z0-9_]*");

	m_operaNameLine->setValidator(new QRegExpValidator(regExp, this));
	if (isOperaTableEmpty()) m_operaDeleteButton->setDisabled(true);/*!< To disable the m_operaDeleteButton if theTrajectoryTable is empty. */
	
        	m_operaD1->setEditable(true);
		m_operaD1->insertItem(0,""); 
		m_operaD2->setEditable(true);
		m_operaD2->insertItem(0,""); 
		m_operaD3->setEditable(true);
		m_operaD3->insertItem(0,""); 

	for (Table<Constant>::reverse_iterator it = localConstantTable->rbegin(); it != localConstantTable->rend(); it++) {
		m_operaD1->insertItem(1, QString(it->getName().c_str()));
		m_operaD2->insertItem(1, QString(it->getName().c_str()));
		m_operaD3->insertItem(1, QString(it->getName().c_str()));
          }

		m_operaTrajname->insertItem(0, "");
	for (Table<Trajectory>::reverse_iterator it = localTrajectoryTable->rbegin(); it != localTrajectoryTable->rend(); it++) 
		m_operaTrajname->insertItem(1, QString(it->getName().c_str()));
	
	modeleOpera                      	= new QStandardItemModel(0, 4) ;
                modeleOpera->setHorizontalHeaderLabels(QStringList() << "Nom Opera" << "Type" << "Dimension(s) " << "Trajectoire"); //Gives names to the columns.

		m_operaTableView->setModel(modeleOpera); /*!< The view takes model.*/
                m_operaTableView->setEditTriggers(QAbstractItemView::NoEditTriggers); /*!< this ligne give no edit mode*/
                m_operaTableView->setShowGrid(false);  /*!< this ligne cover the grille in table */
                m_operaTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
		m_operaTableView->resizeColumnsToContents(); /*!< To optimize the column Opera. */

	//The following to load in the visual Yao the structure of Yao.   
	for (Table<Operator>::iterator it = localOperaTable->begin(); it != localOperaTable->end(); it++) {

	    int row = modeleOpera->rowCount();
            	
	    modeleOpera->setItem(row, 0, new QStandardItem(QString(it->getProperty(0).c_str()))); /*!< Gets the Opera name.*/
	    //modeleOpera->setItem(row, 0, new QStandardItem(QString(it->getType().c_str()))); /*!< Gets the Opera name.*/
            modeleOpera->setItem(row, 2, new QStandardItem(QString(it->getProperty(1).c_str()))); /*!< Gets the Opera dimension (1, 2 or 3).*/
            modeleOpera->setItem(row, 3, new QStandardItem(QString(it->getProperty(2).c_str()))); /*!< Gets the trajectory choosed in the Opera directive.*/
          }

	connect(m_operaAddButton, SIGNAL(clicked()), this, SLOT(addOperaRow())); /*!< Adds rows in the tableview after clicking on the button "Add"*/
   	connect(m_operaDeleteButton, SIGNAL(clicked()), this, SLOT(deleteRow()));
   	connect(m_operaQuitButton, SIGNAL(clicked()), this, SLOT(close())); /*!< Will be closed after a click on the button "Quit" */
   	connect(m_operaHelpButton, SIGNAL(clicked()), this, SLOT(callOperaHelp())); /*!< Calls the help method.*/
    	
   	setFixedWidth(baseSize().width() + 505);
}
//------------------------------------  End of the contructor declaration.---------------------------------------
/*************************************  Implementation of function addOperaRow *************************************************/
void OperaWindow::addOperaRow() {

	int row = modeleOpera->rowCount();//To have the current row of the tableView.

	string name   	= m_operaNameLine->text().toStdString(),
        d1   		= m_operaD1->currentText().toStdString(),
        d2   		= m_operaD2->currentText().toStdString(),
	d3   		= m_operaD3->currentText().toStdString(),
	dim		,//= d1 + " " + d2 + " " + d3,
	traj 		= m_operaTrajname->currentText().toStdString();
	
	vector <int> theShape; 	

	Constant *c1; //Nous créons d'abord un pointeur de constante.
	Constant *c2; //Nous créons d'abord un pointeur de constante.
	Constant *c3; //Nous créons d'abord un pointeur de constante.
 
	c1  = localConstantTable->find(d1); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
	c2 = localConstantTable->find(d2); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
	c3 = localConstantTable->find(d3); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
	
	for (Table<Operator>::iterator it = localOperaTable->begin(); it != localOperaTable->end(); it++)
		if (it->getName() == name) {
			QMessageBox::critical(this, "Avertissement", "Ce nom d'operateur existe déjà.");
			toInitialize();
			return;
		}


	
	//Control on the first dimension's value. 
	if (c1 != 0) { //C'est alors une constante.

		dim = localConstantTable->find(d1)->getText();	
		theShape.push_back(toInt(localConstantTable->find(d1)->getText()));
		
	}
	else {


		for(unsigned int i=0; i < d1.size(); i++) {
		if(!(d1[i] >= '0' && d1[i] <= '9')){
 
                     QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"Dimension 1\"."));
                      return;
                 }

		}

		dim = d1;	
		theShape.push_back(toInt(d1));

	}

	//Control on the 2nd dimension's value. 
	if (c2 != NULL) { //C'est alors une constante.

		dim =  dim + " " + localConstantTable->find(d2)->getText();	
		theShape.push_back(toInt(localConstantTable->find(d2)->getText()));
	
	}
	else {
	
		for(unsigned int i=0; i < d2.size(); i++) {
		if(!(d2[i] >= '0' && d2[i] <= '9')){
 
                     QMessageBox::about(this, tr("Operator erreur de saisie."), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"Dimension 2\"."));
                      return;
                 }

		}

		dim = dim + " " + d2;	
		theShape.push_back(toInt(d2));


	}

	//Control on the 3rd dimension's value. 
	 if (c3 != NULL) { //C'est alors une constante.

		dim =  dim + " " + localConstantTable->find(d3)->getText();	
		theShape.push_back(toInt(localConstantTable->find(d3)->getText()));
	}
	else {
		for(unsigned int i=0; i < d3.size(); i++) {
		if(!(d3[i] >= '0' && d3[i] <= '9')){
 
                     QMessageBox::about(this, tr("Operator: erreur de saisie."), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"Dimension 3\"."));
                      return;
                 }

		}

		dim = dim + " " + d3;	
		theShape.push_back(toInt(d3));
	}
	//Priority between d2 and d3 dimension. 
	if (!d3.empty() && d2.empty())
                                {       
                                         QMessageBox::about(this, tr("Erreur de saisie!"), tr("<p> Veuillez renseigner le champ \"dimension 2\"."));
                                          return;
                                 
                                }
	

	if (name.empty()) { 
		QMessageBox::about(this, tr("Operator: erreur de saisie."),
                     tr("<p>Le champ \"Nom opera\" ne doit pas etre vide."));
              	return;
	}
	else
 
	 if (!(dim.empty() || traj.empty())) {

		modeleOpera->setItem(row, 0, new QStandardItem(m_operaNameLine->text()));
            	modeleOpera->setItem(row, 2, new QStandardItem(QString(dim.c_str())));
		modeleOpera->setItem(row, 3, new QStandardItem(m_operaTrajname->currentText()));
		
		if (!d1.empty()) {
			//Operator Opera(name, localTrajectoryTable->find(traj), theShape);
			//Operator::Type M;
			localOperaTable->push_back(Operator(name, localTrajectoryTable->find(traj), theShape, Operator::H));	
//			emit operaRowIsAdded(true);
		}
		toInitialize();
 
		if (!isOperaTableEmpty()) m_operaDeleteButton->setEnabled(true);
	}
}

/***********************************************************************************************************************/
const int OperaWindow::toInt(const string& s) {
 
     //QStringList test;
      return atoi(s.c_str());
 
}

/************************************* Implementation of function addRow **************************************************/
//void OperaWindow::addRow() {

	//int row = m_operaTableView->rowCount();
	//m_operaTableView->insertRow(row);

//}

/************************************** Implementation of function deleteRow **********************************************/
void OperaWindow::deleteRow() {

    	//m_operaTableView->removeRow(m_operaTableView->currentRow());
    	
	QItemSelectionModel *selection = m_operaTableView->selectionModel();
         index = selection->currentIndex();
         QVariant element = modeleOpera->data(index, Qt::DisplayRole);
          int r = index.row();
          if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
           {
                  modeleOpera->removeRows(m_operaTableView->currentIndex().row(), 1);
                  localOperaTable->erase(localOperaTable->begin() + r);
 
          }
	
	if (isOperaTableEmpty()) m_operaDeleteButton->setDisabled(true);
}

/**********************************************************************/
bool OperaWindow::isOperaTableEmpty() { 

	if (localOperaTable->empty())
		return true;
	else
		return false;
}
/************************************** Implementation of function saveInYaoStructure() ***********************************/
void OperaWindow::saveInYaoStructure() {

	//for (QStringList::iterator it = m_localData.begin(); it != m_localData.end(); it++)
	//	m_operaData.push_back(*it);
	m_operaData = m_localData;
}

/****************************************************************************************************************************/

/****************************************************************************************************************************/
void OperaWindow::toInitialize() {
	
	m_operaNameLine->setText("");
	m_operaD1->setCurrentIndex(0);
	m_operaD2->setCurrentIndex(0);
	m_operaD3->setCurrentIndex(0);
	m_operaTrajname->setCurrentIndex(0);

}
/****************************************************************************************************************************/
bool OperaWindow::isString(string aString) {
//if (aString.length() > 1) {
	bool stringTest = false;
 for (int cpt = 0; aString.size(); cpt++) { 
   if((aString[cpt] >= 'a' && aString[cpt] <= 'z') || (aString[cpt] >= 'A' && aString[cpt] <= 'Z') || (aString[cpt] >= '0' && aString[cpt] <= '9') ||
      (aString[cpt] == '/') || (aString[cpt] == '_') || (aString[cpt] == '.') || (aString[cpt] == ' '))
	stringTest =  true;
   else
   	stringTest =  false;        
	}
//}   
//else
//	{
//	QMessageBox::about(this, tr("Erreur de saisie!"),
  //                             tr("<p>Chaine trop courte!<p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));
    //         //return;
      //  }
	return stringTest;

}


/****************************************************************************************************************************/
bool OperaWindow::isInt(string aStringToInt) {
 
	bool intTest = false;

 for (int cpt = 0; aStringToInt.size(); cpt++) { 
   if(aStringToInt[cpt] >= '0' && aStringToInt[cpt] <= '9')
	intTest = true;
   else
   	intTest = false;        
	}
	
	return intTest;
}
/********************************************* Implémentation du slot callOperaHelp ***************************************/
void OperaWindow::callOperaHelp() {

    QMessageBox::about(this, tr("Aide sur la directive Opera."),("<p> La directive Opera permet de déclarer un eOpera qui sera identifié par son nom (spacename) et de sa taille qui peut être en 1, 2 ou 3 dimensions."
                                                                  "<p><u>Exemple 1</u>: Opera Soce M SZX SZY Toce <p> Pour plus d'informations sur la directive Opera, <a href=\"help/Opera_aide.html\">cliquez ici.</a>"));

}

/****************************************************************************************************************************/
/************************************************************************/
/************************************************************************/void OperaWindow::closeEvent(QCloseEvent *event) {

	emit operaWindowIsClosed(6);
}
