/*!
 * \file    OperaWindow.hpp
 * \brief   Opera class for visual_YAO.
 * \version 2010/06/24 (yyyy/mm/dd)
 * \author  MAYAKI Abdouramane.
 */


#ifndef OPERAWINDOW_HPP
#define OPERAWINDOW_HPP

#include "../../ui_operawindow.h"
#include <QStandardItemModel>
#include <QStringList>
#include <string> 
#include <QMessageBox>
#include "../../../YAOObjects/Component.hpp"
#include "../../../YAOObjects/Constant.hpp"
#include "../../../YAOObjects/Operator.hpp"
#include "../../../YAOObjects/Table.hpp"
#include "../../../YAOObjects/Trajectory.hpp"
#include "../../../YAOObjects/Identifier.hpp"

/*!
 * \class OperaWindow
 * \brief The header file of the OperaWindow class for visual_YAO.
 */


class OperaWindow : public QWidget, Ui::OperaWindow {

    Q_OBJECT                                    /*!< Must be declared for using signals and slots.*/

	public :
            /*!
             *  \brief Constructor
             *
             *  Constructor of the class OperaWindow
             *
             *  \param Table<Opera> theOperaTable
             */


		OperaWindow(yao::Table <yao::Operator> *aOperaTable, yao::Table <yao::Trajectory> *aTrajectory, yao::Table <yao::Constant> *aConstantTable, QWidget *parent=0);     /*!< This function sets the file's name. */

	    	void toInitialize(); // Est appelée pour réinitialiser les champs.
   		bool isString(std::string aString); // Teste si le parametre est une chaine.
	    	bool isInt(std::string aStringToInt);//Teste si le parametre est un entier.
		//void addRow();                      /*!< This function is called to add row in the visual window model. */
		const int toInt(const std::string& s); 
		//typedef yao::Operator Opera; 
		bool isOperaTableEmpty();
		            
	public slots:                               /*!< Functions called after a click. */

		void addOperaRow();
		void deleteRow();                   /*!< This function is called to delete row from the Yao model. */
		void saveInYaoStructure();          /*!< This function is called to add data in the Yao model. */
		void callOperaHelp();
		//void operaRowIsAdded(bool);

	signals:
		void operaWindowIsClosed(int anInt);

	protected:
		void closeEvent(QCloseEvent *event);

	private :

		QModelIndex index;
		QStandardItemModel *modeleOpera;


		QStringList m_horizontalHeader;
		QStringList m_operaData;
		QStringList m_localData;
		//OperaController m_spc;
		yao::Table <yao::Trajectory> *localTrajectoryTable;
		yao::Table <yao::Constant> *localConstantTable;
		yao::Table <yao::Operator> *localOperaTable;               //!< Table of so-called Operas (model operators).

};
#endif // OPERAWINDOW_HPP
