/*!
 * \file    SpaceWindow.hpp
 * \brief   Space class for visual_YAO.
 * \version 2010/06/16 (yyyy/mm/dd)
 * \author  MAYAKI Abdouramane.
 */


#ifndef SPACEWINDOW_HPP
#define SPACEWINDOW_HPP

#include "../../ui_spacewindow.h"
//#include "include/Entete.hpp"
//#include "TrajectoryWindow.hpp"
#include "../../../YAOObjects/Component.hpp"
#include "../../../YAOObjects/Constant.hpp"
#include "../../../YAOObjects/Operator.hpp"
#include "../../../YAOObjects/Table.hpp"
#include "../../../YAOObjects/Trajectory.hpp"
#include "../../../YAOObjects/Identifier.hpp"

class QStandardItemModel;
class QMessageBox;

/*!
 * \class SpaceWindow
 * \brief The header file of the SpaceWindow class for visual_YAO.
 */


class SpaceWindow : public QWidget, Ui::SpaceWindow {

    Q_OBJECT                                    /*!< Must be declared for using signals and slots.*/

	public :
            /*!
             *  \brief Constructor
             *
             *  Constructor of the class SpaceWindow
             *
             *  \param Table<Space> theSpaceTable
             */


		SpaceWindow(yao::Table <yao::Space> *aSpaceTable, yao::Table <yao::Trajectory> *aTrajectory, yao::Table <yao::Constant> *aConstantTable, QWidget *parent=0);     /*!< This function sets the file's name. */

	    	void toInitialize(); // Est appelée pour réinitialiser les champs.
   		bool isString(std::string aString); // Teste si le parametre est une chaine.
	    	bool isInt(std::string aStringToInt);//Teste si le parametre est un entier.
		//void addRow();                      /*!< This function is called to add row in the visual window model. */
		const int toInt(const std::string& s); 
		//typedef yao::Operator Space; 
		bool isSpaceTableEmpty();
		            
	public slots:                               /*!< Functions called after a click. */

		void addSpaceRow();
		void deleteRow();                   /*!< This function is called to delete row from the Yao model. */
		void saveInYaoStructure();          /*!< This function is called to add data in the Yao model. */
		void callSpaceHelp();

	signals:
		void spaceWindowIsClosed(int anInt);
		void spaceRowIsAdded(bool);

	protected:
		void closeEvent(QCloseEvent *event);

	private :

		QModelIndex index;
		QStandardItemModel *modeleSpace;


		QStringList m_horizontalHeader;
		QStringList m_spaceData;
		QStringList m_localData;
		//SpaceController m_spc;
		yao::Table <yao::Trajectory> *localTrajectoryTable;
		yao::Table <yao::Constant> *localConstantTable;
		yao::Table <yao::Space> *localSpaceTable;               //!< Table of so-called spaces (model operators).

};
#endif // SPACEWINDOW_HPP
